using Bukdu
using Estapir
using StructDatabaseMapping
using SQLite
using Test

struct Author <: Model
    id::DBId{Integer}
    name::String
end

struct Book <: Model
    id::DBId{Integer}
    title::String
end
function Book(;
        id::Union{Integer, Nothing} = nothing,
        title::String="")
    return Book(id, title)
end
mapper = DBMapper(()->SQLite.DB("test.db"))
register!(mapper, Author)
register!(mapper, Book)
create_table(mapper, Author)
create_table(mapper, Book)


routes() do
    Estapir.resources(Book, mapper)
end

@testset "Field list" begin
    fields = Estapir.fieldlist(mapper, Book)
    @test fields == [:title]
end
@testset "API" begin
    Bukdu.start(8080)
    @testset "Create Failed: Invalid Request" begin
        payload = JSON.json(
            Dict(
                "username"=>"some_user",
                "password"=>"some_pass"))
                result = Bukdu.Router.call(post,
            "/book",
            ["Content-Type" => "application/json"],
            payload)

        @test result.resp.status == 400
        response = JSON.parse(String(result.resp.body))
        @test response["status"] == Estapir.INVALID_REQUEST
        @test response["missing_fields"] == ["title"]
        @test response["invalid_fields"] == ["password", "username"]
    end
    @testset "Create Successful" begin
        payload = JSON.json(Dict("title"=>"Un titulazo"))
        result = Bukdu.Router.call(post,
            "/book",
            ["Content-Type" => "application/json"],
            payload)
        headers = Dict(result.resp.headers)
        @test haskey(headers, "Location")
        @test result.resp.status == 201
        response = JSON.parse(String(result.resp.body))
        @test response["title"] == "Un titulazo"        
        id = response["id"]["x"]

        result = Bukdu.Router.call(get, "/book/$(id)")
        response = JSON.parse(String(result.resp.body))
        @test response["title"] == "Un titulazo"
        @test response["id"]["x"] == id
    end
    @testset "List" begin
        result = Bukdu.Router.call(get,
            "/book")
        response = JSON.parse(String(result.resp.body))
        @test isa(response, Array)
        println(response)

    end

end

